import unittest
from docs import X4_parser as X4
from docs import X4_threshold as X4T
from docs import TI_parser as TI


class TestParser(unittest.TestCase):

    def test_iq(self):
        """
        Method to test if .dat binary file was converted successfully to .csv file with in-phase and quadrature
        components together.
        """
        file_iq = X4.iq_data('X4data.dat','X4iq')
        self.assertEqual(file_iq, 'converted')

    def test_raw(self):
        """
        Method to test if .dat binary file was converted successfully to .csv file with in-phase and quadrature
        component separated.
        """
        file_raw = X4.raw_data('X4data.dat','X4raw')
        self.assertEqual(file_raw, 'converted')

    def test_TI(self):
        """
        Method to test if .bin binary file was converted successfully to .csv file with iq data put together.
        """

        file_TI = TI.readTIdata('TIdata.bin','TIiq')
        self.assertEqual(file_TI,'converted')

    def X4_Threshold_range_finder(self):
        """
        Method to test if correct range bin was gotten from running function on csv file.
        """

        range_bin = X4T.range_finder("Heli150040.csv",0.02)
        self.assertEqual(range_bin, [9])

    def X4_Threshold_bin_to_distance(self):
        """
        Method to test if range bin to distance converted correctly
        """

        bin_to_distance = X4T.distance_finder("Heli150040.csv", 0.02)
        self.assertEqual(bin_to_distance, [29.25])

    def X4_Threshold_noise_estimate(self):
        """
        Method to test if noise estimate was calculated properly
        """

        noise_estimate = X4T.noise_power_estimate("Heli150040.csv", 0.02)
        self.assertEqual(noise_estimate, 0.00045988029076158947)


if __name__ == '__main__':
    unittest.main()



import numpy as np
import matplotlib.pyplot as plt
import csv


def csv_into_list(filename):
    """
    Converts data from a CSV file into a numpy array.

    Data is collected from "XEP_X4M200_X4M300_plot_record_playback.py". Then
    .dat file is converted into a CSV file using the library.

    Parameters:

        filename: str
            Csv file name.

    Example:

        >>> csv_into_list("Heli150040.csv")
        >>> array([[2.12019056e-04, 2.66481591e-04, 3.51781533e-04, ...,
            2.51499279e-04, 2.63311858e-04, 2.49837321e-04],
           [1.36839763e-03, 1.13654408e-03, 7.92290507e-04, ...,
            3.49299137e-04, 1.30311682e-04, 1.87399805e-04],
           [7.57068081e-04, 6.54611670e-04, 6.83445863e-04, ...,
            6.95117789e-05, 2.17249015e-04, 3.85946482e-04],
           ...,
           [9.57032957e-04, 5.99047943e-04, 5.37280418e-04, ...,
            2.95999810e-04, 2.42217122e-04, 3.25605109e-04],
           [2.37110777e-03, 1.98986895e-03, 1.28061167e-03, ...,
            2.95363991e-04, 1.50057104e-04, 2.33757752e-04],
           [6.45687293e-04, 5.61334516e-04, 1.63285784e-04, ...,
            6.64671904e-05, 2.17249015e-04, 1.93614790e-04]])

    Returns:

        A numpy array of the data.
    """

    with open(filename, "rt") as csvfile:

        data = list(csv.reader(csvfile))  # Declarations
        mean = [0] * len(data[0])

        for i in range(len(data)):  # Goes through the 180 by n CSV file

            for j in range(len(data[i])):
                data[i][j] = complex(data[i][j])  # Converts string to complex number
                mean[j] += complex(
                    data[i][j])  # Adds up numbers for each 180 column to get to get a mean for each column

        for i in range(len(data)):

            for j in range(len(data[i])):
                data[i][j] = abs(data[i][j] - mean[j] / len(
                    data))  # Each complex number is subtracted by it's respective column mean to even it out and the magnitude is taken since it is complex

    data = np.array(data[0:])
    csvfile.close()

    return data


def range_finder(filename, estimated_threshold):
    """
    Finds the range bin/bins from the radar data.

    Given the filename, the file is converted to an array. The points above the given threshold are returned.

    Parameters:

        filename: str
            Csv file name

        estimated_threshold: float
            Threshold to block out background noise.

    Example:

        >>> range_finder("Heli150040.csv",0.02)
        >>> [9]

    Returns:

        A list of range bins that has signal strength values above the threshold.
    """

    data = csv_into_list(filename)
    positive_finds = []

    for i in range(len(data[5])):  # Finds points for the 5th sample

        if (estimated_threshold < data[5][i]):
            positive_finds.append(i + 1)

    return positive_finds


def noise_power_estimate(filename, estimated_threshold):
    """
    Finds the noise power estimate.

    Given the filename, the file is converted to an array. The average for the the array is taken excluding the points above the threshold and their respective guard cells.
    The noise power estimate is then found by subtracting the positive and guard cells from the overall sum. When the noise power estimate is multiplied by the threshold factor, the threshold can be found.

    Parameters:

        filename: str
            Csv file name.

        estimated_threshold: float
            Threshold to block out background noise.

    Example:

        >>> noise_power_estimate("Heli150040.csv",0.02)
        >>> 0.00045988029076158947

    Returns:

        A number representing the noise power estimate.
    """

    data = csv_into_list(filename)
    positive_finds = range_finder(filename, estimated_threshold)

    sum_data = sum(data[5])

    for i in range(len(positive_finds)):  # Subtracts the positive/guard cells from the overall sum
        sum_data -= data[5][positive_finds[i]]
        sum_data -= data[5][positive_finds[i] - 1]
        sum_data -= data[5][positive_finds[i] + 1]

    noise_power = sum_data / (
                len(data[5]) - len(positive_finds) * 3)  # Takes average excluding guard and positive numbers

    return noise_power


def distance_finder(filename, estimate_threshold):
    """
    Converts the positive range bin/bins to a given distance in centimetres for a target.
    Formula used for this is (bin)*5.25-18. Each range bin is 5.25 cm and starting offset is 18 cm.

    Parameters:

        filename: str
            Csv file name.

        estimated_threshold: float
            Threshold to block out background noise.

    Example:

        >>> distance_finder("Heli150040.csv",0.02)
        >>> [29.25]

    Returns:

        List of ranges in centimetres where all the possible targets above the threshold are located.
    """

    positive_finds = range_finder(filename, estimate_threshold)
    distance = []

    for i in range(len(positive_finds)):
        distance.append(positive_finds[i] * 5.25 - 18)

    return distance


def plot_data(filename):
    """
    Plots the data array for the 5th sample set.

    Parameters:

         filename: str
            Csv file name.

    Example:

        >>> plot_data("Heli150040.csv")

    Returns:

        Graph showing the range bin with respect to their corresponding strength of signal.
    """

    data = csv_into_list(filename)
    plt.plot(data[5])
    plt.xlabel('Range bin')
    plt.ylabel('Signal strength')
    plt.title('Detection of target in range bins with equivalent signal strength')
    plt.show()


# This script is used to read the binary file produced by the DCA1000 and Mmwave Studio
import numpy as np
import pandas as pd


def readTIdata(filename,csvname):
    """
    Reads in a binary file and outputs the iq complex data to a csv file specified by csvname.

    Parameter:

        filename: str
            file name of binary file.

        csvname: str
            csv file name that stores the iq data from binary file.

    Example:

        >>> readTIdata('TIdata.bin','TIdata')
        >>> 'converted'

    Return:

        Readable csv file containing complex data.
    """
    # global variables
    # change based on sensor config
    numADCSamples = 256 # number of ADC samples per chirp
    numADCBits = 16 # number of ADC bits per sample
    numRX = 4 # number of receivers
    numLanes = 2 # do not change. number of lanes is always 2
    isReal = False # set to True if real only data, False if complex data
    # read file
    # read .bin file
    with open(filename, 'rb') as f:
        adcData = np.fromfile(f, dtype='int16', count=-1)
        adcData = np.transpose([adcData])
        # if 12 or 14 bits ADC per sample compensate for sign extension
        if numADCBits != 16:
            l_max = 2**(numADCBits-1)-1
            # If value greater than l_max, this loop prevents it
            for index, val in enumerate(adcData):
                if adcData[index] > l_max:
                    adcData[index] -= 2**(numADCBits)
        fileSize = len(adcData)
        # real data reshape, filesize = numADCSamples*numChirps

        if isReal:
            numChirps = int(fileSize/numADCSamples/numRX)
            LVDS = np.zeros((1, fileSize), dtype='int16')
            # each row is data from one chirp
            LVDS = np.reshape(adcData, (numChirps, numADCSamples*numRX))
        else:
            # for complex data
            # filesize = 2 * numADCSamples*numChirps
            numChirps = int(fileSize/2/numADCSamples/numRX)
            LVDS = np.zeros((1, int(fileSize/2)), dtype='complex')
            # combine real and imaginary part into complex data
            # read in file: 2I is followed by 2Q
            counter  = 0
            for i in range(0, fileSize-2, 4):
                # LVDS[0, i] = adcData[i, 0]
                # LVDS[0, i+1] = adcData[i+2, 0]
                LVDS[0, counter] = complex(adcData[i], adcData[i+2])
                LVDS[0, counter+1] = complex(adcData[i+1], adcData[i+3])
                counter += 2
            # each row is data from one chirp
            LVDS = np.reshape(LVDS, (numChirps, numADCSamples*numRX))
            # organize data per RX
            adcData = np.zeros((numRX, numChirps * numADCSamples), dtype='complex')
            for row in range(0, numRX):
                for i in range(0, numChirps):
                    adcData[row, i * numADCSamples:(i + 1) * numADCSamples] = LVDS[i, row * numADCSamples:(row + 1) * numADCSamples]

            data = pd.DataFrame(adcData)
            data.to_csv(csvname+'.csv', index=False, header=False, mode='w')
    f.close()
    return 'converted'


#!/usr/bin/env python
from __future__ import print_function, division

from optparse import OptionParser
from time import sleep

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.animation import FuncAnimation

import pymoduleconnector
from pymoduleconnector import DataType

__version__ = 3


def reset(device_name):
    """
    Resets the device profile and restarts the device

    Parameter:

        device_name: str
            Identifies the device being used for recording using it's port number.
    """
    mc = pymoduleconnector.ModuleConnector(device_name)
    xep = mc.get_xep()
    xep.module_reset()
    mc.close()
    sleep(3)


def on_file_available(data_type, filename):
    """
    Returns the file name that is available after recording.

    Parameters:

        data_type: str
            data type of the recording file.
        filename: str
            file name of recording file.
    """
    print("new file available for data type: {}".format(data_type))
    print("  |- file: {}".format(filename))
    if data_type == DataType.FloatDataType:
        print("processing Float data from file")


def on_meta_file_available(session_id, meta_filename):
    """
    Returns the meta file name that is available after recording.

    Parameters:

        session_id: str
            unique id to identify meta file
        filename: str
            file name of meta file.
    """
    print("new meta file available for recording with id: {}".format(session_id))
    print("  |- file: {}".format(meta_filename))


def clear_buffer(mc):
    """
    Clears the frame buffer

    Parameter:

        mc: object
            module connector object

    """
    xep = mc.get_xep()
    while xep.peek_message_data_float():
        xep.read_message_data_float()


def simple_xep_plot(device_name, record=False, baseband=False):
    """
    Plots the recorded data.

    Parameters:

        device_name: str
            port that device is connected to.
        record: boolean
            check if device is recording.
        baseband: boolean
            check if recording with baseband iq data.

    Return:

        Simple plot of range bin by amplitude.
    """
    FPS = 10
    directory = '.'
    reset(device_name)
    mc = pymoduleconnector.ModuleConnector(device_name)

    # Assume an X4M300/X4M200 module and try to enter XEP mode
    app = mc.get_x4m300()
    # Stop running application and set module in manual mode.
    try:
        app.set_sensor_mode(0x13, 0)  # Make sure no profile is running.
    except RuntimeError:
        # Profile not running, OK
        pass

    try:
        app.set_sensor_mode(0x12, 0)  # Manual mode.
    except RuntimeError:
        # Maybe running XEP firmware only?
        pass

    if record:
        recorder = mc.get_data_recorder()
        recorder.subscribe_to_file_available(pymoduleconnector.AllDataTypes, on_file_available)
        recorder.subscribe_to_meta_file_available(on_meta_file_available)

    xep = mc.get_xep()
    # Set DAC range
    xep.x4driver_set_dac_min(900)
    xep.x4driver_set_dac_max(1150)

    # Set integration
    xep.x4driver_set_iterations(16)
    xep.x4driver_set_pulses_per_step(26)

    xep.x4driver_set_downconversion(int(baseband))
    # Start streaming of data
    xep.x4driver_set_fps(FPS)

    def read_frame():
        """Gets frame data from module"""
        d = xep.read_message_data_float()
        frame = np.array(d.data)
        # Convert the resulting frame to a complex array if downconversion is enabled
        if baseband:
            n = len(frame)
            frame = frame[:n // 2] + 1j * frame[n // 2:]
        return frame

    def animate(i):
        if baseband:
            line.set_ydata(abs(read_frame()))  # update the data
        else:
            line.set_ydata(read_frame())
        return line,

    fig = plt.figure()
    fig.suptitle("example version %d " % (__version__))
    ax = fig.add_subplot(1, 1, 1)
    ax.set_ylim(0 if baseband else -0.03, 0.03)  # keep graph in frame (FIT TO YOUR DATA)
    frame = read_frame()
    if baseband:
        frame = abs(frame)
    line, = ax.plot(frame)

    clear_buffer(mc)

    if record:
        recorder.start_recording(DataType.BasebandApDataType | DataType.FloatDataType, directory)

    ani = FuncAnimation(fig, animate, interval=FPS)
    try:
        plt.show()
    finally:
        # Stop streaming of data
        xep.x4driver_set_fps(0)


def playback_recording(meta_filename, baseband=False):
    """
    Plays back the recording.

    Parameters:

        meta_filename: str
            Name of meta file.
        baseband: boolean
            Check if recording with baseband iq data.
    """
    print("Starting playback for {}".format(meta_filename))
    player = pymoduleconnector.DataPlayer(meta_filename, -1)
    dur = player.get_duration()
    mc = pymoduleconnector.ModuleConnector(player)
    xep = mc.get_xep()
    player.set_playback_rate(1.0)
    player.set_loop_mode_enabled(True)
    player.play()

    print("Duration(ms): {}".format(dur))

    def read_frame():
        """Gets frame data from module"""
        d = xep.read_message_data_float()
        frame = np.array(d.data)
        if baseband:
            n = len(frame)
            frame = frame[:n // 2] + 1j * frame[n // 2:]
        return frame

    def animate(i):
        if baseband:
            line.set_ydata(abs(read_frame()))  # update the data
        else:
            line.set_ydata(read_frame())
        return line,

    fig = plt.figure()
    fig.suptitle("Plot playback")
    ax = fig.add_subplot(1, 1, 1)
    frame = read_frame()
    line, = ax.plot(frame)
    ax.set_ylim(0 if baseband else -0.03, 0.03)  # keep graph in frame (FIT TO YOUR DATA)
    ani = FuncAnimation(fig, animate, interval=10)
    plt.show()

    player.stop()


def main():
    """
    Creates a parser with subcatergories.

    Return:

        A simple XEP plot of live feed from X4 radar.
    """
    parser = OptionParser()
    parser.add_option(
        "-d",
        "--device",
        dest="device_name",
        help="device file to use",
        metavar="FILE")
    parser.add_option(
        "-b",
        "--baseband",
        action="store_true",
        default=False,
        dest="baseband",
        help="Enable baseband, rf data is default")
    parser.add_option(
        "-r",
        "--record",
        action="store_true",
        default=False,
        dest="record",
        help="Enable recording")
    parser.add_option(
        "-f",
        "--file",
        dest="meta_filename",
        metavar="FILE",
        help="meta file from recording")

    (options, args) = parser.parse_args()
    if not options.device_name:
        if options.meta_filename:
            playback_recording(options.meta_filename,
                               baseband=options.baseband)
        else:
            parser.error("Missing -d or -f. See --help.")
    else:
        simple_xep_plot(options.device_name, record=options.record,
                        baseband=options.baseband)


if __name__ == "__main__":
    main()



import numpy as np
import csv


def iq_data(filename,csvname):
    """
    Reads in a binary file and data from range bins is taken and complex iq data is stored in a csv file specified by csvname.

    Parameter:

        filename: str
            The .dat binary file name.
        csvname: str
            User defined .csv file name

    Example:

        >>> iq_data('X4data.dat','X4iq_data')
        >>> 'converted'

    Returns:

        Readable csv file containing complex values.
    """
    with open(filename, "rb") as f:
        data = np.fromfile(f, dtype=np.float32)
    for i in range(0, len(data) // 363 - 1):
        temp = data[3 * (i + 1) + 360 * i:3 * (i + 1) + 360 * (i + 1)]
        iqdata = []
        for j in range(0, 180):
            if temp[j + 180] > 0:
                iqdata.append(str(round(temp[j], 4)) + "+" + str(round(temp[j + 180], 4)) + "j")
            else:
                iqdata.append(str(round(temp[j], 4)) + str(round(temp[j + 180], 4)) + "j")
        with open(csvname+'.csv', 'a', newline="") as csvFile:
             writer = csv.writer(csvFile)
             writer.writerow(iqdata)
    f.close()
    csvFile.close()
    return 'converted'


def raw_data(filename,csvname):
    """
    Reads in a binary file and data from range bins is taken and raw data is stored in a csv file specified by csvname.

    Parameters:

        filename: str
            The .dat binary file name.
        csvname: str
            User defined .csv file name

    Example:

        >>> raw_data('X4data.dat','X4raw_data')
        >>> 'converted'

    Returns:

        Readable csv files containing raw data.
    """
    with open(filename, "rb") as f:
        data = np.fromfile(f, dtype=np.float32)
    for i in range(0, len(data) // 1473 - 1):
        temp = data[3 + 1470 * i:3 + 1470 * (i + 1)]

        with open(csvname+'.csv', 'a', newline="") as csvFile:
            writer = csv.writer(csvFile)
            writer.writerow(temp)
    f.close()
    csvFile.close()
    return 'converted'
